use vvs_llvm::prelude::*;
use vvs_parser::prelude::ast::{self, TypeArgument};
use vvs_runtime_types::{types as vvrt, vvll::LLVMExported as _};

pub(crate) struct TypeLowerer<'a> {
    context: &'a Context<'a>,
}

impl<'a> TypeLowerer<'a> {
    pub fn new(context: &'a Context) -> Self {
        Self { context }
    }

    pub fn lower(self, ty: &ast::TypeInfo) -> Type<'a> {
        let c = self.context.as_ptr();
        match ty {
            ast::TypeInfo::Basic(name) => match name.token_type().as_str() {
                "number" => todo!(),
                "line" => unsafe { Type::from_ptr(vvrt::VVRTLine::llvm_type(c)) },
                "syllabe" => unsafe { Type::from_ptr(vvrt::VVRTSyllabe::llvm_type(c)) },
                "table" | "lines" | "syllabes" => unsafe { Type::from_ptr(vvrt::VVRTTable::llvm_type(c)) },
                ty => unreachable!("unknown type '{ty}'"),
            },
            ast::TypeInfo::Optional { .. } => self.context.type_ptr(),
            ast::TypeInfo::Callback { arguments, return_type, .. } => FunctionType::from((
                TypeLowerer::new(self.context).lower_iter(arguments.iter().map(TypeArgument::type_info)),
                self.lower(return_type),
            ))
            .into(),
            ast::TypeInfo::Table { .. } | ast::TypeInfo::Array { .. } | ast::TypeInfo::Tuple { .. } => {
                todo!("implement arrays, tuples, static tables")
            }
            _ => unreachable!(),
        }
    }

    #[allow(dead_code)]
    pub fn lower_array<const N: usize>(self, tys: [ast::TypeInfo; N]) -> [Type<'a>; N] {
        tys.map(|ty| TypeLowerer::new(self.context).lower(&ty))
    }

    pub fn lower_slice(self, tys: &[&ast::TypeInfo]) -> Vec<Type<'a>> {
        self.lower_iter(tys.iter().copied())
    }

    pub fn lower_iter<'b>(self, tys: impl Iterator<Item = &'b ast::TypeInfo>) -> Vec<Type<'a>> {
        tys.map(|ty| TypeLowerer::new(self.context).lower(ty)).collect()
    }
}
