use std::{env, fs, path::Path};

fn rerun_directory<T: AsRef<Path> + ?Sized>(dir: &T) {
    println!("cargo:rerun-if-changed={}", dir.as_ref().to_string_lossy());
    for entry in std::fs::read_dir(dir).unwrap() {
        let path = entry.expect("Couldn't access file in src directory").path();
        if path.is_dir() {
            rerun_directory(&path);
        }
    }
}

static EXCLUDED_FONTS: &[&str] = &["FiraCode"];

fn main() {
    let out_dir = Path::new(&env::var_os("OUT_DIR").expect("no OUT_DIR env variable..."))
        .canonicalize()
        .expect("failed to canonicalize OUT_DIR");
    let font_dir = Path::new(env!("CARGO_MANIFEST_DIR"))
        .join("../../../rsc/fonts")
        .canonicalize()
        .expect("failed to canonicalize the font folder");

    let fonts = fs::read_dir(&font_dir)
        .expect("failed to read the font folder")
        .filter_map(Result::ok)
        .filter(|file| file.file_type().map(|ft| ft.is_file()).unwrap_or_default())
        .filter(|file| {
            let path = file.path();
            path.file_name()
                .map(|f| !EXCLUDED_FONTS.iter().any(|pat| f.to_string_lossy().starts_with(pat)))
                .unwrap_or_default()
                && path.extension().map(|e| e == "ttf").unwrap_or_default()
        })
        .map(|file| {
            format!(
                "{{ const FONT: &[u8] = include_bytes!({path:?}); ({name:?}, FONT) }}",
                path = file.path().display(),
                name = file.file_name().to_string_lossy().rsplit_once('.').unwrap().0
            )
        })
        .collect::<Vec<_>>()
        .join(",\n");

    fs::write(
        out_dir.join("generated_font_utils.rs"),
        format!(
            r#"
pub fn embeded_fonts() -> impl Iterator<Item = (&'static str, &'static [u8])> {{
    [ {fonts} ].into_iter()
}}
            "#
        ),
    )
    .expect("failed to write generated source file");

    rerun_directory(&font_dir);
    println!("cargo:rerun-if-changed=build.rs");
}
