use crate::passes::declare_pass;
use vvs_parser::prelude::ast::*;
use vvs_shortstring::ShortString;

declare_pass! {
    /// The option optimization is here to substitute the options with the values chosen by the
    /// user.
    #[allow(dead_code)]
    transform::OptsTransform: 'a {}

    aux {
        options: &'a OptionTable,
        setted: &'a [(ShortString, ShortString, TypeInfo, Expression)]
    }
    result { unused:  Vec<(ShortString, ShortString)> }
}

impl VisitorMut for OptsTransform<'_> {
    fn visit_expression_end(&mut self, expression: Expression) -> Expression {
        match expression {
            Expression::Var(Var::Expression(var)) => self.process_var_expression(var),
            expression => expression,
        }
    }
}

impl OptsTransform<'_> {
    fn process_var_expression(&mut self, var: Box<VarExpression>) -> Expression {
        let module = match var.prefix() {
            Prefix::Name(name) if self.options.has_section(name.token_type().as_str()) => name.token_type().as_str(),
            _ => return Expression::Var(Var::Expression(var)),
        };

        match var.suffixes().next() {
            Some(Suffix::Index(Index::Dot { name, .. })) if var.suffixes().count() == 1 => self
                .options
                .get(module, name.token_type().as_str())
                .cloned()
                .unwrap_or_else(|| {
                    self.error(
                        var.tokens().next().unwrap(),
                        format!("failed to find option `{module}.{}`", name.token_type()),
                    );
                    Expression::Symbol(TokenReference::basic_symbol("nil"))
                }),

            Some(Suffix::Index(name)) => {
                self.error(name.tokens().next().unwrap(), "expected a reference to an option, e.g. `module.option`");
                Expression::Symbol(TokenReference::basic_symbol("nil"))
            }

            _ => Expression::Var(Var::Expression(var.clone())),
        }
    }
}
