use crate::prelude::*;
use llvm_sys::{core::*, prelude::*};
use std::marker;

crate::bindings::declare! { mut LLVMContextRef as Context<'a> {
    borrowed: bool,
} }

macro_rules! get_type {
    ($llvm:ident as $name:ident) => {
        paste::paste! {
            #[doc = "Get the type for " $llvm "."]
            pub fn [< type_ $name >](&self) -> Type {
                unsafe { Type::from_ptr([< LLVM $llvm TypeInContext >](self.inner)) }
            }
        }
    };
}

impl Default for Context<'_> {
    fn default() -> Self {
        Self::new()
    }
}

impl Context<'_> {
    /// Create a new context from scratch. This will automatically initialize LLVM if needed.
    pub fn new() -> Self {
        if let Err(err) = crate::init::initialize_llvm() {
            // Should fail only on re-init. If a real problem occured, we would have already
            // aborted the program.
            log::debug!("{err}");
        }
        Self { inner: unsafe { LLVMContextCreate() }, borrowed: false, marker: marker::PhantomData }
    }

    /// Get the pointer type. We always use the address space 0 for now, we will see latter if we
    /// need different address spaces.
    pub fn type_ptr(&self) -> Type {
        unsafe { Type::from_ptr(LLVMPointerTypeInContext(self.inner, 0)) }
    }

    get_type! {  Int8 as   i8 }
    get_type! { Int32 as  i32 }
    get_type! { Float as  f32 }
    get_type! {  Void as void }
}

impl Drop for Context<'_> {
    fn drop(&mut self) {
        if !self.borrowed {
            unsafe { LLVMContextDispose(self.inner) }
        }
    }
}
