#[inline]
pub fn min<T: PartialOrd>(a: T, b: T) -> T {
    crate::either!(a < b => a; b)
}

#[inline]
pub fn max<T: PartialOrd>(a: T, b: T) -> T {
    crate::either!(a > b => a; b)
}

#[macro_export]
macro_rules! min {
    ($x: expr) => { $x };
    ($x: expr, $($z: expr),+) => { ::core::cmp::min($x, $crate::min!($($z),*)) };
}

#[macro_export]
macro_rules! max {
    ($x: expr) => { $x };
    ($x: expr, $($z: expr),+) => { ::core::cmp::max($x, $crate::max!($($z),*)) };
}

#[macro_export]
macro_rules! minmax {
    ($($xs:expr),+) => {(
        $crate::min!($($xs),+),
        $crate::max!($($xs),+),
    )};
}

#[macro_export]
macro_rules! min_partial {
    ($x:expr) => { $x };
    ($x:expr, $($xs:expr),+) => { $crate::min($x, $crate::min_partial!( $($xs),+ )) };
}

#[macro_export]
macro_rules! max_partial {
    ($x:expr) => { $x };
    ($x:expr, $($xs:expr),+) => { $crate::max($x, $crate::max_partial!( $($xs),+ )) };
}

#[macro_export]
macro_rules! minmax_partial {
    ($($xs:expr),+) => {(
        $crate::min_partial!($($xs),+),
        $crate::max_partial!($($xs),+),
    )};
}
