#ifndef VIVY_TIMING_PARAMS_H
#define VIVY_TIMING_PARAMS_H

#ifndef __cplusplus
#error "This is a C++ header"
#endif

#include "PreCompiledHeaders.hh"
#include "Lib/Utils.hh"
#include "TimingScene.hh"

#define getTimingScene() static_cast<TimingScene *>(scene())
#define getTimingParam() getTimingScene()->getParams()

#define Z_SPECTER             -1000
#define Z_AXIS                10
#define Z_LINE_BACKGROUND     -100
#define Z_SEPARATOR_START_END 1000
#define Z_SEPARATOR_MIDDLE    100
#define Z_LINE_SYL_TEXT       50
#define Z_CURSOR_BAR          500

namespace Vivy
{
class TimingParams final : public QWidget {
    Q_OBJECT

public:
    explicit TimingParams(QWidget * = nullptr) noexcept;

private:
    std::unique_ptr<QSlider> zoomSlider;
    std::unique_ptr<QSlider> sensibilitySlider;
    std::unique_ptr<QPushButton> rebuildSceneButton;

public:
    QSlider *getZoomSlider() const noexcept;
    QSlider *getSensibilitySlider() const noexcept;
    QPushButton *getRebuildSceneButton() const noexcept;

private:
    // Managed by TimingAxis
    int m_majorTicks{1}, m_minorTicks{1};
    // Managed by TimingScene
    int m_axisHeight{1}, m_audioWidth{1}, m_audioLength{1}, m_audioHeight{1};
    // Self-managed
    qreal m_wl{0}, m_lw{0};
    qreal m_audioWidthScale{1};

public:
    void adjustTranslate(QRectF *, qreal, qreal) noexcept;
    void adjustFlip(QRectF *, qreal, qreal) noexcept;

    QString printMajorTicks(int) noexcept;
    QString printCursor(int) noexcept;

    inline void setTicks(int t, int T) noexcept
    {
        m_minorTicks = t;
        m_majorTicks = T;
    }
    inline void setAxisHeight(int x) { m_axisHeight = x; }
    inline void setAudioHeight(int x) { m_audioHeight = x; }
    inline void setAudioWidth(int x)
    {
        m_audioWidth = int(x * m_audioWidthScale);
        updateRatios();
    }
    inline void setAudioLength(int x)
    {
        m_audioLength = x;
        updateRatios();
    }

    inline int axisHeight() { return m_axisHeight; }
    inline int audioHeight() { return m_audioHeight; }
    inline int audioWidth() { return m_audioWidth; }
    inline int audioLength() { return m_audioLength; }

    inline void setAudioWidthScale(int s)
    {
        m_audioWidthScale = 1 + 0.01 * s;
        setAudioWidth(m_audioWidth);
        updateRatios();
    }

    template <typename T> T posFromMs(T t) noexcept {
        return T(t * m_wl);
    }
    template <typename T> T posFromMs(T t, int audioWidth, int audioLength) noexcept {
        return audioLength == 0 ? 0 : T(qint64(t) * qint64(audioWidth) / audioLength);
    }

    template <typename T> T msFromPos(T x) noexcept {
        return T(x * m_lw);
    }
    template <typename T> T msFromPos(T x, int audioLength, int audioWidth) noexcept {
        return audioWidth == 0 ? 0 : int(qint64(x) * qint64(audioLength) / audioWidth);
    }

private:
    void updateRatios() noexcept;

signals:
    void paramsChanged();
};

}

#endif // VIVY_TIMING_PARAMS_H
