#pragma once

#include "PreCompiledHeaders.hh"
#include "Lib/Utils.hh"

//  auto model          = new PropertyModel(doc);
//  QVBoxLayout *layout = new QVBoxLayout(this);
//  QTreeView *view     = new QTreeView(this);
//  view->setModel(model);
//  view->header()->setSectionResizeMode(QHeaderView::ResizeToContents);
//  view->expandAll();
//  layout->addWidget(view);
//  setLayout(layout);

namespace Vivy
{
// PropertyModel is used to store a json document and be able to render it
// through a QTreeView
class PropertyModel final : public QAbstractItemModel {
    Q_OBJECT
    VIVY_UNMOVABLE_OBJECT(PropertyModel)

private:
    // Simple property item to be stored inside the PropertyModel
    class Item final {
        VIVY_UNMOVABLE_OBJECT(Item)

    public:
        Item(Item *parent = nullptr) noexcept;
        ~Item() noexcept;

        void appendChild(Item *) noexcept;
        Item *getChild(int row) const noexcept;
        const QVector<Item *> getChilds() const noexcept;
        Item *getParent() const noexcept;
        int getChildCount() const noexcept;
        int getRow() noexcept;
        void setKey(const QString &) noexcept;
        void setValue(const QString &) noexcept;
        void setType(const QJsonValue::Type &) noexcept;
        QString getKey() const noexcept;
        QString getValue() const noexcept;
        QJsonValue::Type getType() const noexcept;

        static Item *fromJson(const QJsonValue &, Item *parent = nullptr);

    private:
        static bool isOrderedJsonObject(QJsonValue const &) noexcept;
        static Item *createChild(Item *root, QJsonObject::ConstIterator const &) noexcept;
        static Item *createChild(Item *root, QJsonObject const &) noexcept;
        static Item *createChild(Item *root, int pos, QJsonValue const &) noexcept;

        QString key{};
        QString value{};
        QJsonValue::Type type{ QJsonValue::Null };
        QVector<Item *> childs{};
        Item *parent{ nullptr };
    };

public:
    explicit PropertyModel(QObject *parent = nullptr) noexcept;
    explicit PropertyModel(const QJsonDocument &, QObject *parent = nullptr);
    explicit PropertyModel(const PropertyConstViewable auto &object, QObject *parent = nullptr)
        : PropertyModel(object.getProperties(), parent)
    {
        // With concepts it seems that the function must be implemented where
        // it is declared.
        root->setKey(object.getElementName());
    }

    [[nodiscard]] static QTreeView *getView(QWidget *parent, PropertyModel *) noexcept;

    QVariant data(const QModelIndex &, int role) const noexcept override;
    bool setData(const QModelIndex &, const QVariant &v, int r = Qt::EditRole) noexcept override;

    QVariant headerData(int section, Qt::Orientation, int role) const noexcept override;
    QModelIndex parent(const QModelIndex &) const noexcept override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const noexcept override;

    int rowCount(const QModelIndex &parent = QModelIndex()) const noexcept override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const noexcept override;

    Qt::ItemFlags flags(const QModelIndex &) const noexcept override;

private:
    void loadJson(const QJsonDocument &json);

    std::unique_ptr<Item> root{ nullptr };
    QStringList headers{};
};
}
